## Deprecated plugins

{%- for dist_name, package_doc in packages.items() %}
### Plugin project {{ dist_name }}

?> :bookmark: This documentation is valid for version `{{ package_doc["dist_version"] }}` of `{{ dist_name}}`. 

{% if package_doc["doc"] is not none %}   
{{ package_doc["doc"] }}
{% endif %} 

---

{% for plugin in package_doc["plugins"] if plugin["deprecated"] or plugin["has_deprecated_arguments"] %}

<div class="plugin-doc">

##### {{ plugin["name"] }}
{% if plugin["deprecated"] %}
> :warning: {{ plugin["deprecation_text_short"] }}

{{ plugin["deprecation_text_long"] }}

{% elif plugin["has_deprecated_arguments"] %}
> :warning: At least one argument has a deprecation warning.
{% endif %}

<!-- tabs:start -->
{% if plugin["description"] is not none %}   

<!-- tab:Description -->

{{ plugin["description"] }}

{% endif %} 

<!-- tab:Arguments -->

{% for arg, arg_info in plugin["arg_info"].items() %}
* {% if arg_info["deprecated"] %}:warning: {% endif %}**`{{ arg }}`:** {{ arg_info["description"] }}

*{{"default = " if "default_value" in arg_info else ""}}{{ arg_info["default_value"] | tojson if "default_value" in arg_info else "" }}{{", " if "default_value" in arg_info else ""}}{{ "Required" if arg_info["required"] else "Optional" }}{{ ", type " + arg_info["typehint_string"] | string if "typehint_string" in arg_info }}*

{% if arg_info["deprecated"] %}>{% if arg_info["deprecation_message"] == "" %}Certain values for the argument have been deprecated and might soon not be accepted anymore. See function below for details.{% else %}{{ arg_info["deprecation_message"] }}{% endif %}
{% endif %}
---
{% endfor %}

{% if plugin["deprecation_check_code"] != "" %} 
Function checking for deprecations:
```python
{{ plugin["deprecation_check_code"] }}
```
---
{% endif %}
How to use in YAML config file:
```yaml
    - {{ plugin["name"] }}:
    {%- for arg, arg_info in plugin["arg_info"].items() %}
        {{ arg }}: {{ arg_info["default"] | tojson if "default" in arg_info else "" }} # {{ "Required" if arg_info["required"] else "Optional" }}{{ ", type " + arg_info["typehint_string"] | string if "typehint_string" in arg_info }}.    
    {%- endfor %}
```

{% if plugin["data_input"] is not none %}   

<!-- tab:Data input -->

{{ plugin["data_input"] }}

{% endif %} 

<!-- tabs:end -->

</div>

{% endfor %}
{% endfor %}
